/*
**   Copyright (c) Steven G. Judd 1994
**		       All Rights Reserved.
**
** Unpublished rights reserved  under the copyright laws of the
** United States.
**++
**
** FACILITY      : Message Queues
** MODULE NAME   : mqexampl.c
**
** ABSTRACT      : MQ Example (Windows NT)
**
**                 MQEXAMPL is a comprehensive example peogram that calls all of 
**				   the functions of the MQ IPC facility for Windows NT.
**
** AUTHOR        : Steven G. Judd
** CREATION DATE : 8-May-1994
**
** MODIFICATION HISTORY :
**
**--
*/

#include <windows.h>
#include <stdlib.h>
#include <stdio.h>

#include <mq_pub.h>
#include <mq_err.h>

/*
** format_message: utility routine illustrating how to use the FormatMessage
** Win32 call to convetr an MQ error code into explanatory text. 
*/

void format_message(long msgid)
{
	char  cBuffer[256];
	DWORD dwStatus;

	dwStatus = FormatMessage(FORMAT_MESSAGE_FROM_HMODULE,
						     GetModuleHandle("mq.dll"), 
						     msgid, 
						     MAKELANGID(LANG_ENGLISH,SUBLANG_ENGLISH_US),
						     cBuffer, sizeof(cBuffer),NULL);

	if (dwStatus > 0)
		printf("%s\n",cBuffer);
	else 
		printf("Error:%d\n",GetLastError());
		
}

/*
** EnumQ: example callback routine for use with MqEnumQ
*/

void
EnumQ(char *qname)
{
	printf("\n");
	printf("Queue: %s\n",qname);
}

/*
** GlobalInfo: example routine for obtaining global MQ system
** information
*/

void GlobalInfo()
{
	MqGlobalInfo ginfo;
	long status;

	status = MqGetGlobalInfo(&ginfo);
	printf("\n        Max Q        :%d\n",ginfo.max_q);
	printf("        Max Msg      :%d\n",ginfo.max_msg);
	printf("        Max Msg Size :%d\n",ginfo.max_msg_size);
	printf("        Max Segments :%d\n",ginfo.max_seg);
	printf("        Segment Size :%d\n",ginfo.seg_size);
	printf("        Free Queues  :%d\n",ginfo.free_q);
	printf("        Free Messages:%d\n",ginfo.free_msg);
	printf("        Free Segments:%d\n",ginfo.free_seg);
	printf("        Free Msg Wait:%d\n",ginfo.wait_msg_count);
	printf("        Free Seg Wait:%d\n",ginfo.wait_seg_count);

}

/*
** QInfo: example routine for obtaining MQ information on a given queue
*/

void QInfo(HMQ hMq)
{
	MqQueueInfo qinfo;
	long status;

	status = MqGetQInfo(hMq,&qinfo);

	printf("        Attach  Count:%d\n",qinfo.attach_count);
	printf("        Message Count:%d\n",qinfo.msg_count);
	printf("        Wait    Count:%d\n",qinfo.wait_count);
	printf("        Destroy      :%d\n",qinfo.destroy);
	printf("        Name         :%s\n",qinfo.q_name);

}

void
main()
{

	
	HMQ hMq=NULL,
		hMq2=NULL;

	long status,lType,
				lFilter,
				lTypeRead,
				lBufSize,
				lBytesRead,
				lBytesToWrite;

	char outbuf[1000],
		 inbuf[1000];

	/*
	** Start by displaying the state of the system
	*/

	printf("\n1.) Display the MQ global state infoprmation.\n\n"); 
	GlobalInfo();

	/*
	** create a queue
	*/

	printf("\n2.) Create a Queue.\n\n"); 
	status = MqCreate(&hMq,"EXAMPLE_Q_1");

	/*
	** display the result of the operation
	*/
	
	format_message(status);

	/*
	** put a message in the queue
	*/

	printf("\n3.) Enqueue a message of type 1.\n\n"); 

	strcpy(outbuf,"This is a simple test message");
	lBytesToWrite = strlen(outbuf);
	lType = 1;

	status = MqPut(hMq,outbuf,lBytesToWrite,lType,MQ_NONBLOCKING,0);

	/*
	** display the result of the operation
	*/
	
	format_message(status);

	/*
	** Display the global info
	*/

	printf("\n4.) Display changed global information.\n\n"); 
	GlobalInfo();

	/*
	** Display the Queue Specific info
	*/

	printf("\n5.) Display queue information.\n\n"); 
	QInfo(hMq);

	/*
	** Try to dequeue a message of a different type, with a 5 sec timeoout
	*/
	
	printf("\n6.) Try to read a message of type 20 with a 5 second timeout.\n\n"); 
	lFilter = 20;
	lBufSize = sizeof(inbuf);

	status = MqGet(hMq,lFilter,&lTypeRead,inbuf,lBufSize,&lBytesRead,MQ_BLOCKING,5000);

	/*
	** display the result of the operation
	*/
	
	format_message(status);

	/*
	**	Try again. accepting ANY message type
	*/

	printf("\n7.) Try to read a message of ANY type with a 5 second timeout.\n\n"); 
	lBufSize = sizeof(inbuf);

	status = MqGet(hMq,MQ_TYPE_ANY,&lTypeRead,inbuf,lBufSize,&lBytesRead,MQ_BLOCKING,5000);

	/*
	** display the result of the operation
	*/
	
	format_message(status);

	/*
	** Make sure the string is terminated
	*/

	inbuf[lBytesRead]=0;

	if (status == MQ_SUCCESS) {
		printf("Read Type:%d Message body:%s\n",lTypeRead, inbuf);
	}

	/*
	** Cancel any blocked get operations
	*/

	printf("\n8.) Cancel any outsntding blocking GET operations.\n\n"); 
	status = MqCancelGet(hMq);

	/*
	** display the result of the operation
	*/
	
	format_message(status);

	/*
	** Cancel any blocked put operations
	*/

	printf("\n9.) Cancel any outsntding blocking PUT operations.\n\n"); 
	status = MqCancelPut(hMq);

	/*
	** display the result of the operation
	*/
	
	format_message(status);

	/*
	**	Establish another attachment to this queue
	*/	

	printf("\n10.) Establish a second attachment to a queue.\n\n"); 
	status = MqAttach(&hMq2,"EXAMPLE_Q_1");

	/*
	** display the result of the operation
	*/
	
	format_message(status);

	/*
	** Display the Queue Specific info again
	*/

	QInfo(hMq);

	/*
	** Mark this queue for removal
	*/

	printf("\n11.) Mark this queue for removal.\n\n"); 
	status = MqDestroy(hMq2);

	/*
	** display the result of the operation
	*/
	
	format_message(status);

	/*
	** Display the Queue Specific info again
	*/

	QInfo(hMq);

	/*
	**	Enumerate all defined queues
	*/

	printf("\n12.) Enumerate all defined queues.\n\n"); 
	status = MqEnumQ(EnumQ);

	/*
	** Detach the last reference to this queue
	*/

	printf("\n13.) Last detach of the queue we created.\n\n"); 

	status = MqDetach(hMq);

	/*
	** display the result of the operation
	*/
	
	format_message(status);

	/*
	** Display the global system state one last time
	*/
	printf("\n14.) Re-display the global information.\n\n"); 

	GlobalInfo();

	return;
}
